#include "wmap.h"
#include "getopt.h"

extern char *optarg;
extern int optint;

static struct option longopts[] = {
    {"help",    no_argument,        0,  'h'},
    {"auto",    no_argument,        0,  'a'},
    {"manual",  no_argument,        0,  'm'},
    {"delay",   required_argument,  0,  'd'},
    {"config",  required_argument,  0,  'c'}
};

static char *shortopts = "hamdc";

/**
 * Console stdout function (for WMap class callback)
 */
void consoleOutput(const char *message) {
    cout << message << endl;
}

/**
 * Console stderr function (for WMap class callback)
 */
void consoleError(const char *message) {
    cout << message << endl;
}

/**
 * Console stdin function (for WMap class callback)
 */
string consoleInput() {
    string result;
    cin >> result;
    return result;
}

int printHelp(const char *programName, const char *error) {
    if (error)
        cout << error << endl;
    cout << "Command line help:" << endl;
    cout << " Automatic Mode:" << endl;
    cout << "  " << programName << " --auto http://domain.com" << endl;
    cout << " Manual Mode:" << endl;
    cout << "  " << programName << " --manual http://domain.com" << endl;
    cout << "Full command option summary:" << endl;
    cout << " " << programName << " {flags} {base URL}" << endl;
    cout << "Flags:" << endl;
    cout << " -a --auto                      Automatic mode" << endl;
    cout << " -m --manual                    Manual mode" << endl;
    cout << " -d {seconds} --delay={seconds} In auto mode, delay between requests" << endl;
    cout << " -c {file} --config={file}      Use {file} as the config file" << endl;
    cout << " -h --help                      Print this information" << endl;
    cout << "One of --auto or --manual is required" << endl;
    cout << "Default delay in auto mode is 3 seconds" << endl;
    return 1;
}

int main(int argc, char **argv) {
    int automode = 0;
    int manualmode = 0;
    int delay = 3;
    int ch;
    string configFile = "";
    string error = "";
    string program = argv[0];
    while ((ch = getopt_long(argc, argv, shortopts, longopts, NULL)) != -1) {
        switch(ch) {
            case 'h':
                return printHelp(program.c_str(), NULL);
            case 'a':
                automode = 1;
                break;
            case 'm':
                manualmode = 1;
                break;
            case 'd':
                if (optarg == 0)
                    return printHelp(program.c_str(), "--delay requires an argument");
                delay = atoi(optarg);
                break;
            case 'c':
                if (optarg == 0)
                    return printHelp(program.c_str(), "--config requires an argument");
                configFile = optarg;
                break;
            case '?':
                error = string("Unknown flag ") + (char) ch;
                return printHelp(program.c_str(), error.c_str());
            default:
                return printHelp(program.c_str(), NULL);
        }
    }
    argc -= optind;
    argv += optind;
    if (!automode && !manualmode)
        return printHelp(program.c_str(), "One of --auto|--manual is required");
    if (automode && manualmode)
        return printHelp(program.c_str(), "Only one of --auto|--manual is allowed");
    if (argc != 1)
        return printHelp(program.c_str(), "Base URL is a required argument");
    
    WMap wmap(argv[0]);
    wmap.setOutputFunction(&consoleOutput);
    wmap.setErrorFunction(&consoleError);
    wmap.setInputFunction(&consoleInput);
    wmap.setDelay(delay);
    if (automode)
        wmap.setRunMode(RUN_AUTOMATIC);
    if (manualmode) {
        cout << "Enter directory/file name to search for at base URL (^C to quit)" << endl;
        wmap.setRunMode(RUN_MANUAL);
    }
    wmap.run();
    return 0;
}